@@ -2,9 +2,11 @@
 from __future__ import annotations
 import logging
 from ssl import SSLError
-import time
 import asyncio
-from typing import Union
+from typing import Callable, Union
+import socket
+from functools import wraps
+import inspect
 
 import aiohttp
 from aiohttp.client_exceptions import ContentTypeError
@@ -18,17 +20,59 @@ from .const import (
     Resolution,
     FPS,
 )
-from .ddp import async_get_status, get_status, wakeup
+from .ddp import async_get_status, get_status, wakeup, STATUS_OK, search, async_search
 from .session import Session
-from .util import get_users, get_profiles, format_regist_key
+from .util import format_regist_key
 from .register import register
 from .controller import Controller
+from .profile import Profiles, UserProfile
 
 
 _LOGGER = logging.getLogger(__name__)
 
-STATUS_OK = 200
-STATUS_STANDBY = 620
+
+def _status_to_device(hosts: list[dict]) -> list[RPDevice]:
+    devices = []
+    for status in hosts:
+        ip_address = status.get("host-ip")
+        if ip_address:
+            device = RPDevice(ip_address)
+            # pylint: disable=protected-access
+            device._set_status(status)
+            devices.append(device)
+    return devices
+
+
+def _load_profiles(func: Callable):
+    """Decorator. Load profiles if profiles is None."""
+
+    @wraps(func)
+    def wrapped(*args, **kwargs):
+        param = "profiles"
+        signature = inspect.signature(func)
+        bound_args = signature.bind(*args, **kwargs)
+        if not isinstance(bound_args.arguments.get("self"), RPDevice):
+            raise TypeError(f"Can only wrap a {RPDevice} instance")
+        if param not in signature.parameters:
+            raise ValueError(f"Method: {func} has no parameter '{param}'")
+
+        profiles = bound_args.arguments.get(param)
+        if not profiles:
+            param_index = list(signature.parameters.keys()).index(param)
+            # Remove profiles from arguments
+            if param_index < len(bound_args.args):
+                _args = list(args)
+                _args.pop(param_index)
+                args = tuple(_args)
+            kwargs[param] = Profiles.load()
+        else:
+            if not isinstance(profiles, Profiles):
+                raise TypeError(
+                    f"Expected {param} to be instance of {Profiles}. Got: {type(profiles)}"
+                )
+        return func(*args, **kwargs)
+
+    return wrapped
 
 
 class RPDevice:
@@ -42,21 +86,44 @@ class RPDevice:
     """
 
     @staticmethod
-    def get_all_users(profiles: dict = None, profile_path="") -> list:
-        """Return all users that have been authenticated with OAuth."""
-        if not profiles:
-            profiles = get_profiles(profile_path)
-        return list(profiles.keys())
+    def get_all_users(profiles: Profiles = None) -> list[str]:
+        """Return all usernames that have been authenticated with OAuth."""
+        profiles = Profiles.load()
+        return profiles.usernames
+
+    @staticmethod
+    def get_profiles(path: str = "") -> Profiles:
+        """Return Profiles.
+
+        :param path: Path to file to load profiles.
+            If not given, will load profiles from default path.
+        """
+        return Profiles.load(path)
+
+    @staticmethod
+    def search() -> list[RPDevice]:
+        """Return all devices that are discovered."""
+        hosts = search()
+        return _status_to_device(hosts)
+
+    @staticmethod
+    async def async_search() -> list[RPDevice]:
+        """Return all devices that are discovered. Async."""
+        hosts = await async_search()
+        return _status_to_device(hosts)
 
     def __init__(self, host: str):
+        socket.gethostbyname(host)  # Raise Exception if invalid
+
         self._host = host
         self._max_polls = DEFAULT_POLL_COUNT
         self._host_type = None
         self._host_name = None
         self._mac_address = None
         self._ip_address = None
+        self._ddp_version = None
+        self._system_version = None
         self._callback = None
-        self._standby_start = 0
         self._unreachable = False
         self._status = {}
         self._media_info = None
@@ -64,58 +131,69 @@ class RPDevice:
         self._session = None
         self._controller = None
 
-    def get_users(self, profiles: dict = None, profile_path="") -> list[str]:
+    @_load_profiles
+    def get_users(self, profiles: Profiles = None) -> list[str]:
         """Return Registered Users."""
         if not self.mac_address:
             _LOGGER.error("Device ID is unknown. Status needs to be updated.")
             return []
-        users = get_users(self.mac_address, profiles, profile_path)
+        users = profiles.get_users(self.mac_address)
         return users
 
-    def get_profile(self, user: str, profiles: dict = None, profile_path="") -> dict:
+    @_load_profiles
+    def get_profile(self, user: str, profiles: Profiles = None) -> UserProfile:
         """Return valid profile for user.
 
-        :param profiles: dict of all user profiles. If None, profiles will be retrieved from default location. Optional.
-        :param profile_path: Path to saved profile file. Specify if profile data was not saved to the default path. Optional.
+        See:
+        :meth:`pyremoteplay.oauth.get_user_account() <pyremoteplay.oauth.get_user_account>`
+        :meth:`pyremoteplay.profile.format_user_account() <pyremoteplay.profile.format_user_account>`
+
+        :param user: Username of user
+        :param profiles: dict of all user profiles.
+            If None, profiles will be retrieved from default location. Optional.
         """
-        if not profiles:
-            profiles = get_profiles(profile_path)
         users = self.get_users(profiles)
         if user not in users:
-            _LOGGER.error("User: %s not valid", user)
+            _LOGGER.error("User: %s not registered", user)
             return None
         return profiles.get(user)
 
     def get_status(self) -> dict:
         """Return status."""
         status = get_status(self.host)
-        self.set_status(status)
+        self._set_status(status)
         return status
 
     async def async_get_status(self):
         """Return status. Async."""
-        status = async_get_status(self.host)
-        self.set_status(status)
+        status = await async_get_status(self.host)
+        self._set_status(status)
         return status
 
     def set_unreachable(self, state: bool):
         """Set unreachable attribute."""
         self._unreachable = state
 
-    def set_callback(self, callback: callable):
+    def set_callback(self, callback: Callable):
         """Set callback for status changes."""
         self._callback = callback
 
-    def set_status(self, data):
+    def _set_status(self, data: dict):
         """Set status."""
-        if self.host_type is None:
+        if not data:
+            return
+        if data.get("host-type") is not None:
             self._host_type = data.get("host-type")
-        if self.mac_address is None:
+        if data.get("host-id") is not None:
             self._mac_address = data.get("host-id")
-        if self.host_name is None:
+        if data.get("host-name") is not None:
             self._host_name = data.get("host-name")
-        if self.ip_address is None:
+        if data.get("host-ip") is not None:
             self._ip_address = data.get("host-ip")
+        if data.get("device-discovery-protocol-version") is not None:
+            self._ddp_version = data.get("device-discovery-protocol-version")
+        if data.get("system-version") is not None:
+            self._system_version = data.get("system-version")
         old_status = self.status
         self._status = data
         if old_status != data:
@@ -129,13 +207,6 @@ class RPDevice:
             if not title_id and self.callback:
                 # Call immediately since we don't have to get media.
                 self.callback()  # pylint: disable=not-callable
-            # Status changed from OK to Standby/Turned Off
-            if (
-                old_status is not None
-                and old_status.get("status-code") == STATUS_OK
-                and self.status.get("status-code") == STATUS_STANDBY
-            ):
-                self._standby_start = time.time()
 
     async def _get_media_info(self, title_id: str, region="United States"):
         """Retrieve Media info."""
@@ -159,8 +230,7 @@ class RPDevice:
     def create_session(
         self,
         user: str,
-        profiles: dict = None,
-        profile_path="",
+        profiles: Profiles = None,
         loop: asyncio.AbstractEventLoop = None,
         receiver: AVReceiver = None,
         resolution: Union[Resolution, str, int] = "360p",
@@ -172,7 +242,7 @@ class RPDevice:
         """Return initialized session if session created else return None.
         Also connects a controller.
 
-        See 'Session' for param details.
+        See :class:`Session <pyremoteplay.session.Session>`  for param details.
 
         :param user: Name of user to use. Can be found with `get_users`
 
@@ -182,7 +252,7 @@ class RPDevice:
                 _LOGGER.error("Device session already exists. Disconnect first.")
                 return None
             self.disconnect()  # Cleanup session
-        profile = self.get_profile(user, profiles, profile_path)
+        profile = self.get_profile(user, profiles)
         if not profile:
             _LOGGER.error("Could not find valid user profile")
             return None
@@ -218,12 +288,16 @@ class RPDevice:
             del self._session
         self._session = None
 
-    async def standby(self, user="", profiles: dict = None, profile_path="") -> bool:
+    async def standby(self, user="", profiles: Profiles = None) -> bool:
         """Place Device in standby. Return True if successful.
 
         If there is a valid and connected session, no arguments need to be passed.
         Otherwise creates and connects a session first.
 
+        If already connected, the sync method
+        :meth:`RPDevice.session.standby() <pyremoteplay.session.Session.standby>`
+        is available.
+
         :param user: Name of user to use. Can be found with `get_users`
         """
         if not self.is_on:
@@ -233,7 +307,7 @@ class RPDevice:
             if not user:
                 _LOGGER.error("User needed")
                 return False
-            self.create_session(user, profiles, profile_path)
+            self.create_session(user, profiles)
         if not self.connected:
             if not await self.connect():
                 _LOGGER.error("Error connecting")
@@ -249,8 +323,7 @@ class RPDevice:
     def wakeup(
         self,
         user: str = "",
-        profiles: dict = None,
-        profile_path: str = None,
+        profiles: Profiles = None,
         key: str = "",
     ):
         """Send Wakeup.
@@ -264,7 +337,7 @@ class RPDevice:
         if not key:
             if not user:
                 raise ValueError("User must be specified")
-            profile = self.get_profile(user, profiles, profile_path)
+            profile = self.get_profile(user, profiles)
             if not profile:
                 _LOGGER.error("Profile not found")
                 return
@@ -272,31 +345,43 @@ class RPDevice:
         regist_key = format_regist_key(key)
         wakeup(self.host, regist_key, host_type=self.host_type)
 
+    @_load_profiles
     def register(
         self,
         user: str,
         pin: str,
         timeout: float = 2.0,
-        profiles: dict = None,
-        profile_path="",
-    ) -> dict:
-        """Register psn_id with device. Return register info.
+        profiles: Profiles = None,
+        save: bool = True,
+    ) -> UserProfile:
+        """Register psn_id with device. Return updated user profile.
 
         :param user: User name. Can be found with `get_all_users`
         :param pin: PIN for linking found on Remote Play Host
         :param timeout: Timeout to wait for completion
+        :param profiles: Profiles to use
+        :param save: Save profiles if True
         """
-        if not profiles:
-            profiles = get_profiles(profile_path)
-        data = profiles.get(user)
-        if not data:
+        if not self.status:
+            _LOGGER.error("No status")
+            return None
+        profile = profiles.get_user_profile(user)
+        if not profile:
             _LOGGER.error("User: %s not found", user)
-            return {}
-        psn_id = data.get("id")
+            return None
+        psn_id = profile.id
         if not psn_id:
             _LOGGER.error("Error retrieving ID for user: %s", user)
-            return {}
-        return register(self.host, psn_id, pin, timeout)
+            return None
+        regist_data = register(self.host, psn_id, pin, timeout)
+        if not regist_data:
+            _LOGGER.error("Registering failed")
+            return None
+        profile.add_regist_data(self.status, regist_data)
+        profiles.update_user(profile)
+        if save:
+            profiles.save()
+        return profile
 
     @property
     def host(self) -> str:
@@ -323,6 +408,16 @@ class RPDevice:
         """Return IP Address."""
         return self._ip_address
 
+    @property
+    def ddp_version(self) -> str:
+        """Return DDP Version."""
+        return self._ddp_version
+
+    @property
+    def system_version(self) -> str:
+        """Return System Version."""
+        return self._system_version
+
     @property
     def remote_port(self) -> int:
         """Return DDP port of device."""
@@ -339,7 +434,7 @@ class RPDevice:
         return self._unreachable
 
     @property
-    def callback(self) -> callable:
+    def callback(self) -> Callable:
         """Return callback for status updates."""
         return self._callback
 
@@ -397,11 +492,6 @@ class RPDevice:
             return True
         return False
 
-    @property
-    def standby_start(self) -> float:
-        """Return timestamp when device was seen changing to standby."""
-        return self._standby_start
-
     @property
     def controller(self) -> Controller:
         """Return Controller."""
