@@ -1,10 +1,12 @@
 """GUI Main Methods for pyremoteplay."""
 import sys
-
+import logging
 from PySide6 import QtCore, QtWidgets
 
 from .main_window import MainWindow
 
+logging.basicConfig(level=logging.INFO)
+
 
 def main():
     """Run GUI."""
@@ -13,7 +15,7 @@ def main():
     )
     app = QtWidgets.QApplication([])
     app.setApplicationName("PyRemotePlay")
-    widget = MainWindow(app)
+    widget = MainWindow()
     widget.resize(800, 600)
     widget.show()
     sys.exit(app.exec())
