@@ -31,7 +31,7 @@ from .const import (
     Quality,
 )
 from .crypt import SessionCipher
-from .ddp import async_get_status, async_wakeup
+from .ddp import async_get_status, wakeup
 from .errors import RemotePlayError, RPErrorHandler
 from .keys import (
     SESSION_KEY_0_PS4,
@@ -41,6 +41,7 @@ from .keys import (
 )
 from .stream import RPStream
 from .util import format_regist_key
+from .profile import UserProfile
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -170,7 +171,7 @@ class Session:
     """Remote Play Session Async.
 
     :param host: IP Address of Remote Play Host
-    :param profile: Profile data to connect with. From registering
+    :param profile: User Profile to connect with
     :param loop: A running asyncio event loop. If None, loop will be the current running loop
     :param receiver: A receiver for handling video and audio frames
     :param resolution: The resolution of video stream.
@@ -244,7 +245,7 @@ class Session:
     def __init__(
         self,
         host: str,
-        profile: dict,
+        profile: Union[dict, UserProfile],
         loop: asyncio.AbstractEventLoop = None,
         receiver: AVReceiver = None,
         resolution: Union[Resolution, str, int] = "360p",
@@ -551,10 +552,10 @@ class Session:
         _LOGGER.info("Sending Standby")
         self.stop()
 
-    async def wakeup(self):
+    def _send_wakeup(self):
         """Wakeup Host."""
         regist_key = format_regist_key(self._regist_key)
-        await async_wakeup(self.host, regist_key, host_type=self.type)
+        wakeup(self.host, regist_key, host_type=self.type)
 
     async def start(self, wakeup=True, autostart=True) -> bool:
         """Start Session/RP Session."""
@@ -576,7 +577,7 @@ class Session:
             return False
         if not status[1]:
             if wakeup:
-                await self.wakeup()
+                self._send_wakeup()
                 self.error = "Host is in Standby. Attempting to wakeup."
             return False
         if not await self._run_io(self._connect):
